/* 
 * File:   Finch.h
 * Author: Tom Lauwers, BirdBrain Technologies (tlauwers@finchrobot.com)
 *
 * Contains the necessary function definitions to run a Finch in C++.
 * For full documentation, view the associated Finch.cpp
 * 
 * Created on May 23, 2011, 5:30 PM
 */

#ifndef FINCH_H
#define	FINCH_H


/***************** for Ch Space *******************/
#ifdef _CH_
#include <dlfcn.h>

class Finch {
    static void *g_Finch_dlhandle;
    static int g_Finch_dlcount;
public:
    Finch();
    virtual ~Finch();
    int connect();
    int disConnect();
    int setLED(int red, int green, int blue);
    int setMotors(...);
    //int setMotors(int leftWheelSpeed, int rightWheelSpeed, int duration);
    int noteOn(...);
    //int noteOn(int frequency, int duration);
    int noteOff();
    double getTemperature();
    double* getAccelerations();
    int* getLightSensors();
    int* getObstacleSensors();
    int wasTapped();
    int wasShaken();
    int isObstacleLeftSide();
    int isObstacleRightSide();
    int getLeftLightSensor();
    int getRightLightSensor();
    double getXAcceleration();
    double getYAcceleration();
    double getZAcceleration();
    int isBeakUp();
    int isBeakDown();
    int isFinchLevel();
    int isFinchUpsideDown();
    int isRightWingDown();
    int isLeftWingDown();
    int counter();
    void keepAlive();  
    int finchRead(unsigned char bufToWrite[], unsigned char bufRead[]);
    int finchWrite(unsigned char bufToWrite[]);
    /* unsigned int __stdcall keepAliveEntryPoint(void * pThis)
      {
          Finch * pthX = (Finch*)pThis;   // Cast from void to Finch class pointer
          pthX->keepAlive();           // now call the true entry-point-function

          return 1;          // the thread exit code
      }
    */
};

void * Finch::g_Finch_dlhandle = NULL;
int Finch::g_Finch_dlcount = 0;

#pragma importf <Finch.chf>
#else

/*************** for C++ space ************************/
class Finch {
public:
    Finch();
    virtual ~Finch();
    int connect();
    int disConnect();
    int setLED(int red, int green, int blue);
    int setMotors(int leftWheelSpeed, int rightWheelSpeed);
    int setMotors(int leftWheelSpeed, int rightWheelSpeed, int duration);
    int noteOn(int frequency);
    int noteOn(int frequency, int duration);
    int noteOff();
    double getTemperature();
    double* getAccelerations();
    int* getLightSensors();
    int* getObstacleSensors();
    int wasTapped();
    int wasShaken();
    int isObstacleLeftSide();
    int isObstacleRightSide();
    int getLeftLightSensor();
    int getRightLightSensor();
    double getXAcceleration();
    double getYAcceleration();
    double getZAcceleration();
    int isBeakUp();
    int isBeakDown();
    int isFinchLevel();
    int isFinchUpsideDown();
    int isRightWingDown();
    int isLeftWingDown();
    int counter();
    void keepAlive();  
    int finchRead(unsigned char bufToWrite[], unsigned char bufRead[]);
    int finchWrite(unsigned char bufToWrite[]);
    static unsigned __stdcall keepAliveEntryPoint(void * pThis)
      {
          Finch * pthX = (Finch*)pThis;   // Cast from void to Finch class pointer
          pthX->keepAlive();           // now call the true entry-point-function

          return 1;          // the thread exit code
      }
private:
};

#endif  /* _CH_ */

#endif	/* FINCH_H */
